@@ -19,7 +19,7 @@
  */
 
 #ifndef lint
-static const volatile char rcsid[] = "@(#)$Id: send.c,v 1.96 2004/12/10 14:39:04 chopin Exp $";
+static const volatile char rcsid[] = "@(#)$Id: send.c,v 1.104 2005/02/22 16:34:02 chopin Exp $";
 #endif
 
 #include "os.h"
@@ -142,42 +142,17 @@ int	send_message(aClient *to, char *msg, int len)
 	    }
 	if (IsDead(to))
 		return 0; /* This socket has already been marked as dead */
-	if (DBufLength(&to->sendQ) > get_sendq(to))
+	if (DBufLength(&to->sendQ) > (i=get_sendq(to, CBurst(to))))
 	{
-# ifdef HUB
-		if (CBurst(to))
+		to->exitc = EXITC_SENDQ;
+		if (IsService(to) || IsServer(to))
 		{
-			/* Allow bursting clients (servers or services) to
-			 * exceed their maxsendq. --B. */
-#  ifdef MAXSENDQ_NOTICES
-			static	time_t	lastnotice = 0;
-
-			/* It's total, not per destination, so it can
-			 * happen some notices escape us. Better this
-			 * than getting 50Hz notices. --B. */
-			if (timeofday - lastnotice >= 60)
-			{
-				sendto_flag(SCH_NOTICE, "Max SendQ limit "
-					"exceeded for %s: %d > %d",
-					to->name,
-					DBufLength(&to->sendQ), get_sendq(to));
-				lastnotice = timeofday;
-			}
-#  endif
-		}
-		else
-# endif
-		{
-			to->exitc = EXITC_SENDQ;
-			if (IsService(to) || IsServer(to))
-			{
-				return dead_link(to,
+			return dead_link(to,
 				"Max SendQ limit exceeded for %s: %d > %d",
-					get_client_name(to, FALSE),
-					DBufLength(&to->sendQ), get_sendq(to));
-			}
-			return dead_link(to, "Max Sendq exceeded");
+				get_client_name(to, FALSE),
+				DBufLength(&to->sendQ), i);
 		}
+		return dead_link(to, "Max Sendq exceeded");
 	}
 # ifdef	ZIP_LINKS
 	/*
@@ -215,7 +190,7 @@ tryagain:
 			/* Anyway, 10% increase. */
 			poolsize *= 1.1;
 			sendto_flag(SCH_NOTICE,
-				    "New poolsize %d. (reached)",
+				    "New poolsize %u. (reached)",
 				    poolsize);
 			istat.is_dbufmore++;
 			goto tryagain;
@@ -361,7 +336,7 @@ int	send_queued(aClient *to)
 static	anUser	ausr;
 static	aClient	anon;
 
-void initanonymous()
+void initanonymous(void)
 {
 	memset(&ausr, 0, sizeof(anUser));
 	strcpy(ausr.username, "anonymous");
@@ -561,28 +536,32 @@ int	sendto_serv_v(aClient *one, int ver, char *pattern, ...)
 		if ((cptr = local[fdas.fd[i]]) &&
 		    (!one || cptr != one->from) && !IsMe(cptr))
 		{
-			if (cptr->serv->version & ver)
-			{
-				if (!len)
-				{
-					va_list	va;
-					va_start(va, pattern);
-					len = vsendprep(pattern, va);
-					va_end(va);
-				}
-
-				(void)send_message(cptr, sendbuf, len);
-			}
-			else
+#if 0
+/* We're not using it for now, so just save some cpu.
+** Revive once we need it --B. */
+			if ((cptr->serv->version & ver) == 0)
 			{
 				rc = 1;
+				continue;
 			}
+#endif
+			if (!len)
+			{
+				va_list	va;
+				va_start(va, pattern);
+				len = vsendprep(pattern, va);
+				va_end(va);
+			}
+			(void)send_message(cptr, sendbuf, len);
 		}
 	}
 
 	return rc;
 }
 
+#if 0
+/* We're not using it for now, so just save some cpu.
+** Revive once we need it --B. */
 int	sendto_serv_notv(aClient *one, int ver, char *pattern, ...)
 {
 	Reg	int	i, len=0, rc=0;
@@ -614,6 +593,7 @@ int	sendto_serv_notv(aClient *one, int ver, char *pattern, ...)
 
 	return rc;
 }
+#endif
 
 /*
  * sendto_common_channels()
@@ -851,11 +831,15 @@ int	sendto_match_servs_v(aChannel *chptr, aClient *from, int ver,
 			continue;
 		if (!BadPtr(mask) && match(mask, cptr->name))
 			continue;
+#if 0
+/* We're not using it for now, so just save some cpu.
+** Revive once we need it --B. */
 		if ((ver & cptr->serv->version) == 0)
 		    {
 			rc = 1;
 			continue;
 		    }
+#endif
 		if (!len)
 		    {
 			va_list	va;
@@ -868,6 +852,9 @@ int	sendto_match_servs_v(aChannel *chptr, aClient *from, int ver,
 	return rc;
 }
 
+#if 0
+/* We're not using it for now, so just save some cpu.
+** Revive once we need it --B. */
 int	sendto_match_servs_notv(aChannel *chptr, aClient *from, int ver,
 		char *format, ...)
 {
@@ -908,15 +895,13 @@ int	sendto_match_servs_notv(aChannel *chptr, aClient *from, int ver,
 	    }
 	return rc;
 }
+#endif
 
 /*
  * sendto_match_butone
  *
  * Send to all clients which match the mask in a way defined on 'what';
  * either by user hostname or user servername.
- * NOTE: we send it only to new servers and local clients. Old servers
- * are covered by sendto_match_butone_old() and upper function takes care
- * of calling both with properly formatted parameters.
  */
 void	sendto_match_butone(aClient *one, aClient *from, char *mask, int what,
 		char *pattern, ...)
@@ -931,11 +916,7 @@ void	sendto_match_butone(aClient *one, aClient *from, char *mask, int what,
 			continue;       /* that clients are not mine */
  		if (cptr == one)	/* must skip the origin !! */
 			continue;
-		/* This used to be IsServer(cptr), but we have to use
-		** another (sendto_match_butone_old()) function to
-		** send $#/$$-mask messages to old servers, which do
-		** not understand new syntax. --Beeth */
-		if (ST_UID(cptr))
+		if (IsServer(cptr))
 		{
 			/*
 			** we can save some CPU here by not searching the
@@ -976,60 +957,6 @@ void	sendto_match_butone(aClient *one, aClient *from, char *mask, int what,
 	return;
 }
 
-/*
- * sendto_match_butone_old
- *
- * Send to all clients which match the mask in a way defined on 'what';
- * either by user hostname or user servername.
- * NOTE: we send it only to old servers (pre-2.11). The rest is
- * covered by sendto_match_butone()
- */
-void	sendto_match_butone_old(aClient *one, aClient *from, char *mask,
-		int what, char *pattern, ...)
-{
-	int	i;
-	aClient *cptr,
-		*srch;
-  
-	for (i = fdas.highest; i >= 0; i--)
-	{
-		if (!(cptr = local[fdas.fd[i]]))
-			continue;	/* not a server */
- 		if (cptr == one)	/* must skip the origin !! */
-			continue;
-		if (IsMe(cptr))
-			continue;
-		/* we want to pick only old servers to perhaps send them
-		** $/#-mask message, if it's old, check if clients match */
-		if (ST_NOTUID(cptr))
-		{
-			/* see comment in sendto_match_butone() */
-			for (srch = cptr->prev; srch; srch = srch->prev)
-			{
-				if (!IsRegisteredUser(srch))
-					continue;
-				if (srch->from == cptr &&
-				    match_it(srch, mask, what))
-					break;
-			}
-			if (srch == NULL)
-				continue;
-		}
-		else
-		{
-			continue;
-		}
-		/* this frame have tricked me many times ;) and it's only
-		** frame for having va declared ;) --Beeth */
-		{
-			va_list	va;
-			va_start(va, pattern);
-			vsendto_prefix_one(cptr, from, pattern, va);
-			va_end(va);
-		}
-	}
-	return;
-}
 /*
 ** sendto_ops_butone
 **	Send message to all operators.
@@ -1084,23 +1011,23 @@ static	void	vsendto_prefix_one(aClient *to, aClient *from, char *pattern,
  * sends a message to a server-owned channel
  */
 static	SChan	svchans[SCH_MAX] = {
-	{ SCH_ERROR,	"&ERRORS",	NULL },
-	{ SCH_NOTICE,	"&NOTICES",	NULL },
-	{ SCH_KILL,	"&KILLS",	NULL },
-	{ SCH_CHAN,	"&CHANNEL",	NULL },
-	{ SCH_NUM,	"&NUMERICS",	NULL },
-	{ SCH_SERVER,	"&SERVERS",	NULL },
-	{ SCH_HASH,	"&HASH",	NULL },
-	{ SCH_LOCAL,	"&LOCAL",	NULL },
-	{ SCH_SERVICE,	"&SERVICES",	NULL },
-	{ SCH_DEBUG,	"&DEBUG",	NULL },
-	{ SCH_AUTH,	"&AUTH",	NULL },
-	{ SCH_SAVE,	"&SAVE",	NULL },
-	{ SCH_WALLOP,	"&WALLOPS",	NULL },
+	{ SCH_ERROR,	"&ERRORS",	NULL, -2},
+	{ SCH_NOTICE,	"&NOTICES",	NULL, -2},
+	{ SCH_KILL,	"&KILLS",	NULL, -2},
+	{ SCH_CHAN,	"&CHANNEL",	NULL, -2},
+	{ SCH_NUM,	"&NUMERICS",	NULL, -2},
+	{ SCH_SERVER,	"&SERVERS",	NULL, -2},
+	{ SCH_HASH,	"&HASH",	NULL, -2},
+	{ SCH_LOCAL,	"&LOCAL",	NULL, -2},
+	{ SCH_SERVICE,	"&SERVICES",	NULL, -2},
+	{ SCH_DEBUG,	"&DEBUG",	NULL, -2},
+	{ SCH_AUTH,	"&AUTH",	NULL, -2},
+	{ SCH_SAVE,	"&SAVE",	NULL, -2},
+	{ SCH_WALLOP,	"&WALLOPS",	NULL, -2},
 #ifdef CLIENTS_CHANNEL
-	{ SCH_CLIENT,	"&CLIENTS",	NULL },
+	{ SCH_CLIENT,	"&CLIENTS",	NULL, -2},
 #endif
-	{ SCH_OPER,	"&OPER",	NULL },
+	{ SCH_OPER,	"&OPER",	NULL, -2},
 };
 
 
@@ -1109,6 +1036,57 @@ void	setup_svchans(void)
 	int	i;
 	SChan	*shptr;
 
+#ifdef LOG_SERVER_CHANNELS
+	/* They're here, as they need to be done only once per server run,
+	** to determine if we want given channel logged. --B. */
+# ifdef LOG_SCH_ERROR
+	(svchans+SCH_ERROR)->fd = -1;
+# endif
+# ifdef LOG_SCH_NOTICE
+ 	(svchans+SCH_NOTICE)->fd = -1;
+# endif
+# ifdef LOG_SCH_KILL
+	(svchans+SCH_KILL)->fd = -1;
+# endif
+# ifdef LOG_SCH_CHAN
+	(svchans+SCH_CHAN)->fd = -1;
+# endif
+# ifdef LOG_SCH_NUM
+	(svchans+SCH_NUM)->fd = -1;
+# endif
+# ifdef LOG_SCH_SERVER
+	(svchans+SCH_SERVER)->fd = -1;
+# endif
+# ifdef LOG_SCH_HASH
+	(svchans+SCH_HASH)->fd = -1;
+# endif
+# ifdef LOG_SCH_LOCAL
+	(svchans+SCH_LOCAL)->fd = -1;
+# endif
+# ifdef LOG_SCH_SERVICE
+	(svchans+SCH_SERVICE)->fd = -1;
+# endif
+# ifdef LOG_SCH_DEBUG
+	(svchans+SCH_DEBUG)->fd = -1;
+# endif
+# ifdef LOG_SCH_AUTH
+	(svchans+SCH_AUTH)->fd = -1;
+# endif
+# ifdef LOG_SCH_SAVE
+	(svchans+SCH_SAVE)->fd = -1;
+# endif
+# ifdef LOG_SCH_WALLOP
+	(svchans+SCH_WALLOP)->fd = -1;
+# endif
+# ifdef CLIENTS_CHANNEL
+#  ifdef LOG_SCH_CLIENT
+	(svchans+SCH_CLIENT)->fd = -1;
+#  endif
+# endif
+# ifdef LOG_SCH_OPER
+	(svchans+SCH_OPER)->fd = -1;
+# endif
+#endif
 	for (i = SCH_MAX - 1, shptr = svchans + i; i >= 0; i--, shptr--)
 		shptr->svc_ptr = find_channel(shptr->svc_chname, NULL);
 }
@@ -1155,7 +1133,16 @@ void	sendto_flag(u_int chan, char *pattern, ...)
 		    }
 #endif
 	    }
+#ifdef LOG_SERVER_CHANNELS
+	if ((svchans+chan)->fd >= 0)
+	{
+		char	lbuf[1024];
+		int	len;
 
+		len = sprintf(lbuf, "%u %s\n", (u_int)timeofday, nbuf);
+		(void)write((svchans+chan)->fd, lbuf, len);
+	}
+#endif
 	return;
 }
 
@@ -1164,6 +1151,32 @@ static int connlog = -1;
 
 void	logfiles_open(void)
 {
+#ifdef LOG_SERVER_CHANNELS
+	int	i;
+	SChan	*shptr;
+	char	fname[BUFSIZE];
+
+	for (i = SCH_MAX - 1, shptr = svchans + i; i >= 0; i--, shptr--)
+	{
+		if (shptr->fd == -2)
+		{
+			/* We don't want this channel logged. */
+			continue;
+		}
+
+		sprintf(fname, "%s.%s", FNAME_SCH_PREFIX, shptr->svc_chname+1);
+		shptr->fd = open(fname, O_WRONLY|O_APPEND|O_NDELAY
+#ifdef LOGFILES_ALWAYS_CREATE
+			|O_CREAT, S_IRUSR|S_IWUSR
+#endif
+			);
+		/* Better safe than sorry. */
+		if (shptr->fd >= 0)
+		{
+			local[shptr->fd] = NULL;
+		}
+	}
+#endif
 #ifdef  FNAME_USERLOG
 	userlog = open(FNAME_USERLOG, O_WRONLY|O_APPEND|O_NDELAY
 # ifdef	LOGFILES_ALWAYS_CREATE
@@ -1195,6 +1208,18 @@ void	logfiles_open(void)
 
 void	logfiles_close(void)
 {
+#ifdef LOG_SERVER_CHANNELS
+	int	i;
+	SChan	*shptr;
+	
+	for (i = SCH_MAX - 1, shptr = svchans + i; i >= 0; i--, shptr--)
+	{
+		if (shptr->fd >= 0)
+		{
+			(void)close(shptr->fd);
+		}
+	}
+#endif
 #ifdef FNAME_USERLOG
 	if (userlog != -1)
 	{
