@@ -24,7 +24,7 @@
 #undef RES_C
 
 #ifndef lint
-static const volatile char rcsid[] = "@(#)$Id: res.c,v 1.42 2004/11/02 16:32:01 chopin Exp $";
+static const volatile char rcsid[] = "@(#)$Id: res.c,v 1.45 2005/02/22 17:24:17 chopin Exp $";
 #endif
 
 /* because there is a lot of debug code in here :-) */
@@ -487,7 +487,7 @@ static	int	do_query_number(Link *lp, struct IN_ADDR *numb, ResRQ *rptr)
 			"%x.%x.%x.%x.%x.%x.%x.%x."
 			"%x.%x.%x.%x.%x.%x.%x.%x."
 			"%x.%x.%x.%x.%x.%x.%x.%x."
-			"%x.%x.%x.%x.%x.%x.%x.%x.ip6.%s.",
+			"%x.%x.%x.%x.%x.%x.%x.%x.ip6.arpa.",
 		(u_int)(cp[15]&0xf), (u_int)(cp[15]>>4),
 		(u_int)(cp[14]&0xf), (u_int)(cp[14]>>4),
 		(u_int)(cp[13]&0xf), (u_int)(cp[13]>>4),
@@ -503,13 +503,7 @@ static	int	do_query_number(Link *lp, struct IN_ADDR *numb, ResRQ *rptr)
 		(u_int)(cp[3]&0xf), (u_int)(cp[3]>>4),
 		(u_int)(cp[2]&0xf), (u_int)(cp[2]>>4),
 		(u_int)(cp[1]&0xf), (u_int)(cp[1]>>4),
-		(u_int)(cp[0]&0xf), (u_int)(cp[0]>>4),
-#ifdef SIXBONE_HACK
-		/* use ip6.arpa for 2001, ip6.int for all the rest
-		   (idea from Axu) --B. */
-		(cp[0] != 0x20 || cp[1] != 0x01) ? "int" :
-#endif
-		"arpa");
+		(u_int)(cp[0]&0xf), (u_int)(cp[0]>>4));
 	    }
 #else
 	cp = (u_char *)&numb->s_addr;
@@ -622,7 +616,7 @@ static	int	proc_answer(ResRQ *rptr, HEADER *hptr, char *buf, char *eob)
 	alias = hp->h_aliases;
 	while (*alias)
 		alias++;
-#if SOLARIS_2 && !defined(__GNUC__) /* brain damaged compiler it seems */
+#if defined(SOLARIS_2) && !defined(__GNUC__) /* brain damaged compiler it seems */
 	for (; hptr->qdcount > 0; hptr->qdcount--)
 #else
 	while (hptr->qdcount-- > 0)
@@ -1645,7 +1639,7 @@ time_t	expire_cache(time_t now)
 /*
  * remove all dns cache entries.
  */
-void	flush_cache()
+void	flush_cache(void)
 {
 	Reg	aCache	*cp;
 
