@@ -22,7 +22,7 @@
  */
 
 #ifndef lint
-static const volatile char rcsid[] = "@(#)$Id: s_service.c,v 1.59 2004/10/01 20:22:16 chopin Exp $";
+static const volatile char rcsid[] = "@(#)$Id: s_service.c,v 1.65 2005/02/22 17:09:37 chopin Exp $";
 #endif
 
 #include "os.h"
@@ -142,7 +142,7 @@ void	check_services_butone(long action, char *server, aClient *cptr,
 				va_end(va);
 				if ((sp->wants & SERVICE_WANT_UID))
 					sendto_one(sp->bcptr, ":%s%s", 
-						HasUID(cptr) ? cptr->user->uid :
+						cptr->user ? cptr->user->uid :
 						cptr->name, buf);
 				else
 					sendto_one(sp->bcptr, ":%s!%s@%s%s",
@@ -172,7 +172,7 @@ static	void	sendnum_toone(aClient *cptr, int wants, aClient *sptr,
 	if (!*umode)
 		umode = "+";
 
-	if ((wants & SERVICE_WANT_UID) && HasUID(sptr))
+	if ((wants & SERVICE_WANT_UID) && sptr->user)
 		sendto_one(cptr, ":%s UNICK %s %s %s %s %s %s :%s",
 			sptr->user->servp->sid,
 			(wants & SERVICE_WANT_NICK) ? sptr->name : ".",
@@ -192,8 +192,8 @@ static	void	sendnum_toone(aClient *cptr, int wants, aClient *sptr,
 			   : ".",
 			   (wants & SERVICE_WANT_USER) ? sptr->user->host :".",
 			   (wants & SERVICE_WANT_USER) ?
-			   ((wants & SERVICE_WANT_TOKEN) ?
-			    sptr->user->servp->tok : sptr->user->server) : ".",
+			   ((wants & SERVICE_WANT_SID) ?
+			    sptr->user->servp->sid : sptr->user->server) : ".",
 			   (wants & (SERVICE_WANT_UMODE|SERVICE_WANT_OPER)) ? umode : "+",
 			   (wants & SERVICE_WANT_USER) ? sptr->info : "");
 	else
@@ -217,8 +217,8 @@ static	void	sendnum_toone(aClient *cptr, int wants, aClient *sptr,
 		if (wants & SERVICE_WANT_USER)
 			sendto_one(cptr, ":%s USER %s %s %s :%s", prefix, 
 				   sptr->user->username, sptr->user->host,
-				   (wants & SERVICE_WANT_TOKEN)?
-				   sptr->user->servp->tok : sptr->user->server,
+				   (wants & SERVICE_WANT_SID)?
+				   sptr->user->servp->sid : sptr->user->server,
 				   sptr->info);
 		if (wants & (SERVICE_WANT_UMODE|SERVICE_WANT_OPER))
 			sendto_one(cptr, ":%s MODE %s %s", prefix, sptr->name,
@@ -324,9 +324,8 @@ int	m_service(aClient *cptr, aClient *sptr, int parc, char *parv[])
 	aConfItem *aconf;
 #endif
 	aServer	*sp = NULL;
-	char	*dist, *server = NULL, *info, *stok;
+	char	*dist, *server = NULL, *info;
 	int	type, i;
-	char	*mlname;
 
 	if (sptr->user)
 	    {
@@ -334,7 +333,7 @@ int	m_service(aClient *cptr, aClient *sptr, int parc, char *parv[])
 		return 1;
 	    }
 
-	if ((ST_NOTUID(cptr) && parc < 7) || parc < 5)
+	if (parc < 5)
 	{
 		sendto_one(cptr, replies[ERR_NEEDMOREPARAMS], ME,
 			   BadTo(parv[0]), "SERVICE");
@@ -342,18 +341,9 @@ int	m_service(aClient *cptr, aClient *sptr, int parc, char *parv[])
 	}
 
 	/* Copy parameters into better documenting variables */
-	if (ST_NOTUID(cptr))
-	{
-		dist = parv[3];
-		type = strtol(parv[4], NULL, 0);
-		info = parv[6];
-	}
-	else
-	{
-		dist = parv[2];
-		type = strtol(parv[3], NULL, 0);
-		info = parv[4];
-	}
+	dist = parv[2];
+	type = strtol(parv[3], NULL, 0);
+	info = parv[4];
 
 	/*
 	 * Change the sender's origin.
@@ -368,17 +358,8 @@ int	m_service(aClient *cptr, aClient *sptr, int parc, char *parv[])
 		
 		/* 2.11 protocol - :SID SERVICE ..
 		 * - we know that the sptr contains the correct server */
-		if (ST_UID(cptr))
-		{
-			acptr->hopcount = sptr->hopcount;
-			sp = sptr->serv;
-		}
-		else
-		{ /* old 2.10 protocol */
-			server = parv[2];
-			acptr->hopcount = atoi(parv[5]);
-			sp = find_tokserver(atoi(server), cptr, NULL);
-		}
+		acptr->hopcount = sptr->hopcount;
+		sp = sptr->serv;
 		
 		if (sp == NULL)
 		{
@@ -518,33 +499,10 @@ int	m_service(aClient *cptr, aClient *sptr, int parc, char *parv[])
 		if (match(dist, bcptr->name))
 			continue;
 
-		if (ST_UID(bcptr))
-		{
-			sendto_one(bcptr, ":%s SERVICE %s %s %d :%s",
-					sp->sid, acptr->name, dist, type, info);
-		}
-		else
-		{
-			mlname = my_name_for_link(ME, bcptr->serv->nline->port);
-		
-			if (*mlname == '*' && match(mlname,
-						acptr->service->server)== 0)
-			{
-				/* Me masking the service to 2.10 */
-				stok = me.serv->tok;
-			}
-			else
-			{	/* Mask masked services */
-				stok = sp->maskedby->serv->tok;
-			}
-			
-			sendto_one(bcptr, "SERVICE %s %s %s %d %d :%s",
-					acptr->name, stok, dist, type,
-					acptr->hopcount+1, info);
-
-		}
+		sendto_one(bcptr, ":%s SERVICE %s %s %d :%s",
+				sp->sid, acptr->name, dist, type, info);
 	}
-	
+		
 	return 0;
 }
 
@@ -638,18 +596,11 @@ int	m_servset(aClient *cptr, aClient *sptr, int parc, char *parv[])
 				continue;
 			split = (MyConnect(acptr) &&
 				 mycmp(acptr->name, acptr->sockhost));
-			if (split)
-				sendto_one(sptr,":%s SERVER %s %d %s :[%s] %s",
-					   acptr->serv->up->name, acptr->name,
-					   acptr->hopcount+1,
-					   acptr->serv->tok,
-					   acptr->sockhost, acptr->info);
-			else
-				sendto_one(sptr, ":%s SERVER %s %d %s :%s",
-					   acptr->serv->up->name, acptr->name,
-					   acptr->hopcount+1,
-					   acptr->serv->tok,
-					   acptr->info);
+			sendto_one(sptr, ":%s SERVER %s %d %s :%s",
+				acptr->serv->up->name, acptr->name,
+				acptr->hopcount+1,
+				acptr->serv->sid,
+				acptr->info);
 		    }
 	    }
 
